@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 // Test these using #if AT_CUDNN_ENABLED(), not #ifdef, so that it's
@@ -18,3 +19,7 @@
 #endif
 
 #define NVCC_FLAGS_EXTRA ""
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
