@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/jit/ir/attributes.h>
@@ -78,9 +79,7 @@ namespace aten {
 using namespace ::c10::aten;
 }
 namespace cuda {
-#if !defined(USE_ROCM)
-using namespace ::c10::hip;
-#endif
+using namespace ::c10::cuda;
 } // namespace cuda
 
 struct Function;
@@ -1831,3 +1830,7 @@ struct FunctionSchemaMap {
 };
 
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
